@@ -37,6 +37,11 @@ POSSIBILITY OF SUCH DAMAGE.
 
 /* License changed to modified BSD by cph on 2004-11-08.  */
 
+/*
+ * Modifications for Scheme 48 by Martin Gasbichler, Mike Sperber,
+ * Marcus Crestani, Harald Glab-Phlak
+ */
+
 /* Changes for Scheme 48:
  *  - Converted to ANSI.
  *  - Added bitwise operations.
@@ -47,7 +52,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "bignum.h"
 #include "bignumint.h"
 #include <limits.h>
-#include "scheme48.h"	/* for S48_GC_PROTECT_GLOBAL */
 #include <stdio.h>
 #include <stdlib.h>	/* abort */
 #include <math.h>
@@ -104,9 +108,9 @@ static void        bignum_negate_magnitude(bignum_type);
 static long        bignum_unsigned_logcount(bignum_type arg);
 static int         bignum_unsigned_logbitp(int shift, bignum_type bignum);
 
-static s48_value s48_bignum_zero    = (s48_value) NULL;
-static s48_value s48_bignum_pos_one = (s48_value) NULL;
-static s48_value s48_bignum_neg_one = (s48_value) NULL;
+static s48_ref_t s48_bignum_zero    = NULL;
+static s48_ref_t s48_bignum_pos_one = NULL;
+static s48_ref_t s48_bignum_neg_one = NULL;
 
 /* Exports */
 
@@ -119,25 +123,22 @@ static s48_value s48_bignum_neg_one = (s48_value) NULL;
 void
 s48_bignum_make_cached_constants(void)
 {
-  if (s48_bignum_zero == (s48_value) NULL) {
+  if (s48_bignum_zero == NULL) {
     bignum_type temp;
     
-    s48_bignum_zero = (s48_value) BIGNUM_ALLOCATE_TAGGED(0);
-    temp = S48_ADDRESS_AFTER_HEADER(s48_bignum_zero, long);
+    s48_bignum_zero = s48_make_global_ref((s48_value) BIGNUM_ALLOCATE_TAGGED(0));
+    temp = S48_ADDRESS_AFTER_HEADER(s48_deref(s48_bignum_zero), long);
     BIGNUM_SET_HEADER (temp, 0, 0);
-    S48_GC_PROTECT_GLOBAL(s48_bignum_zero);
     
-    s48_bignum_pos_one = (s48_value) BIGNUM_ALLOCATE_TAGGED(1);
-    temp = S48_ADDRESS_AFTER_HEADER(s48_bignum_pos_one, long);
+    s48_bignum_pos_one = s48_make_global_ref((s48_value) BIGNUM_ALLOCATE_TAGGED(1));
+    temp = S48_ADDRESS_AFTER_HEADER(s48_deref(s48_bignum_pos_one), long);
     BIGNUM_SET_HEADER (temp, 1, 0);
     (BIGNUM_REF (temp, 0)) = 1;
-    S48_GC_PROTECT_GLOBAL(s48_bignum_pos_one);
     
-    s48_bignum_neg_one = (s48_value) BIGNUM_ALLOCATE_TAGGED(1);
-    temp = S48_ADDRESS_AFTER_HEADER(s48_bignum_neg_one, long);
+    s48_bignum_neg_one = s48_make_global_ref ((s48_value) BIGNUM_ALLOCATE_TAGGED(1));
+    temp = S48_ADDRESS_AFTER_HEADER(s48_deref(s48_bignum_neg_one), long);
     BIGNUM_SET_HEADER (temp, 1, 1);
     (BIGNUM_REF (temp, 0)) = 1;
-    S48_GC_PROTECT_GLOBAL(s48_bignum_neg_one);
   }
 }
 
@@ -481,12 +482,12 @@ s48_bignum_to_long(bignum_type bignum)
     bignum_digit_type * scan = (start + (BIGNUM_LENGTH (bignum)));
     while (start < scan)
       accumulator = ((accumulator << BIGNUM_DIGIT_LENGTH) + (*--scan));
-    return ((BIGNUM_NEGATIVE_P (bignum)) ? (-((long)accumulator)) : accumulator);
+    return ((BIGNUM_NEGATIVE_P (bignum)) ? (-((long)accumulator)) : (long)accumulator);
   }
 }
 
 bignum_type
-ulong_to_bignum(unsigned long n)
+s48_ulong_to_bignum(unsigned long n)
 {
   bignum_digit_type result_digits [BIGNUM_DIGITS_FOR_LONG];
   bignum_digit_type * end_digits = result_digits;
@@ -639,7 +640,7 @@ s48_bignum_length_upper_limit(void)
 
 bignum_type
 s48_digit_stream_to_bignum(unsigned int n_digits,
-			   unsigned int *producer(bignum_procedure_context),
+			   unsigned int (*producer)(bignum_procedure_context),
 			   bignum_procedure_context context,
 			   unsigned int radix,
 			   int negative_p)
@@ -681,8 +682,8 @@ s48_digit_stream_to_bignum(unsigned int n_digits,
 void
 s48_bignum_to_digit_stream(bignum_type bignum,
 			   unsigned int radix,
-			   void *consumer(bignum_procedure_context,
-					  bignum_digit_type),
+			   void (*consumer)(bignum_procedure_context,
+					    bignum_digit_type),
 			   bignum_procedure_context context)
 {
   BIGNUM_ASSERT ((radix > 1) && (radix <= BIGNUM_RADIX_ROOT));
@@ -2010,7 +2011,7 @@ bignum_unsigned_logcount(bignum_type arg)
 
   /* sufficient for any reasonable big number */
   long result;
-  int i;
+  unsigned long i;
 
   if (BIGNUM_ZERO_P (arg)) return (0L);
 
