@@ -18,6 +18,11 @@
  * with the software or the use or other dealings in the software.
  */
 
+/*
+ * Modifications for Scheme 48 by Mike Sperber, Eric Knauel,
+ * David Frese, Taylor Campbell
+ */
+
 #include <stdio.h>
 #include <string.h>
 
@@ -399,9 +404,9 @@ int s48_dragon(buf, v) char *buf; double v; {
    if (e == exp_max) {
      /* infinity or NaN */
      if (f == 0)
-       strcpy(buf, sign ? "#{-Inf}" : "#{Inf}");
+       strcpy(buf, sign ? "-inf.0" : "+inf.0");
      else
-       strcpy(buf, "#{NaN}");
+       strcpy(buf, "+nan.0");
      return 0;
    }
 
