@@ -1,3 +1,9 @@
+/*
+ * Part of Scheme 48 1.9.  See file COPYING for notices and license.
+ *
+ * Authors: Richard Kelsey, Jonathan Rees, Mike Sperber, Marcus Crestani
+ */
+
 /*
  * Externally visible objects defined in scheme48vm.c.
  */
@@ -33,7 +39,7 @@ extern char		s48_external_event_pendingPUunsafe(void);
 extern long		s48_dequeue_external_eventBUunsafe(char*);
 
 /* imported and exported bindings */	  
-S48_EXTERN void		s48_define_exported_binding(char *, s48_value);
+S48_EXTERN s48_value	s48_define_exported_binding(char *, s48_value);
 S48_EXTERN s48_value	s48_get_imported_binding(char *);
 
 /* for raising exceptions in external code */
@@ -41,6 +47,8 @@ extern void		s48_setup_external_exception(s48_value exception,
 						     long nargs);
 extern s48_value	s48_resetup_external_exception(s48_value exception,
 						       long additional_nargs);
+extern void		s48_argument_type_violation(s48_value value);
+extern void		s48_range_violation(s48_value value, s48_value min, s48_value max);
 extern void		s48_push(s48_value value);
 extern s48_value	s48_stack_ref(long offset);
 extern void		s48_stack_setB(long offset, s48_value value);
@@ -49,6 +57,16 @@ extern void		s48_stack_setB(long offset, s48_value value);
 
 S48_EXTERN s48_value	s48_allocate_string(long l);
 
+extern s48_value s48_enter_string_utf_16beU(char *);
+extern s48_value s48_enter_string_utf_16be_nU(char *, long);
+extern long s48_copy_string_to_utf_16beU(s48_value, char *);
+extern long s48_copy_string_to_utf_16be_nU(s48_value, long, long, char *);
+
+extern s48_value s48_enter_string_utf_16leU(char *);
+extern s48_value s48_enter_string_utf_16le_nU(char *, long);
+extern long s48_copy_string_to_utf_16leU(s48_value, char *);
+extern long s48_copy_string_to_utf_16le_nU(s48_value, long, long, char *);
+
 /* called when writing an image */
 extern s48_value	s48_symbol_table(void);
 extern s48_value	*s48_channels(void);
@@ -56,9 +74,6 @@ extern long		s48_channel_count(void);
 extern s48_value	s48_imported_bindings(void);
 extern s48_value	s48_exported_bindings(void);
 
-/* called when resuming an image */
-extern psbool		s48_warn_about_undefined_imported_bindings(void);
-
 /* for initializion on SMP machines */
 extern void		s48_initialize_shared_registersB(long, long, long, long);
 
@@ -69,15 +84,21 @@ extern s48_value	s48_really_add_channel(s48_value, s48_value, long);
 
 /* external allocation and GC roots */
 extern void		s48_gc_root(void);
+extern long		s48_gc_run_time(long *);       
 extern s48_value	s48_allocate_stob(long type, long size);
+extern s48_value	s48_allocate_weak_stob(long type, long size);
+extern s48_value	s48_allocate_unmovable_stob(long type, long size);
+extern psbool		s48_unmovableP(s48_value);
+extern psbool		s48_gc_can_allocate_unmovableP(void);
 S48_EXTERN void		s48_push_gc_rootsB(char *, long);
 S48_EXTERN psbool	s48_pop_gc_rootsB(void);
 extern char *		s48_set_gc_roots_baseB(void);
 extern psbool		s48_release_gc_roots_baseB(char *);
-S48_EXTERN char *	s48_register_gc_rootB(char *marker);
+S48_EXTERN char *	s48_register_gc_rootB(char *);
 S48_EXTERN void		s48_unregister_gc_rootB(char *);
 extern void		s48_reset_external_rootsB(void);
 extern void		s48_post_gc_cleanup(psbool, psbool);
+extern void		s48_trace_external_calls(void);
 
 /* for native code */
 extern void		s48_copy_stack_into_heap();
