@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey.  See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, David Frese, Marcus Crestani, Mike Sperber
 
 (config '(load "../scheme/vm/macro-package-defs.scm"))
 (load-package 'vm-architecture)
@@ -6,7 +8,8 @@
 (in 'simplify-let '(run (set! *duplicate-lambda-size* 15)))
 (in 'prescheme-compiler
     '(run (prescheme-compiler
-	   '(vm external-gc-roots interpreter-gc)
+	   '(vm external-gc-roots interpreter-gc
+		io-opcodes) ; message-element
 	   '("../scheme/platform-interfaces.scm"
 	     "../scheme/vm/ps-platform-64-packages.scm"
 	     "../scheme/vm/interfaces.scm"
@@ -19,7 +22,8 @@
 	   "../scheme/vm/scheme48vm-64.c"
 	   '(header "#include \"scheme48vm-prelude.h\"")
 	   '(copy (interpreter pop-continuation-from-stack
-			       env-and-template-setup))
+			       env-and-template-setup)
+		  (text-encodings encode-scalar-value decode-scalar-value))
 	   '(no-copy (interpreter interpret
 				  application-exception
 				  handle-interrupt
