@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey.  See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Mike Sperber
 
 
 ; Data must be done last as it may contain references to the other stuff.
@@ -176,6 +178,8 @@
 	 (desyntaxify thing))
 	((primop? thing)
 	 (primop-id thing))
+	((operator? thing)
+	 (operator-name thing))
 	((primitive? thing)
 	 (primitive-id thing))
 	((variable? thing)
