@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber
 
 ; Alternate implementations of the low-structures.
 ; Cf. low-structures-interface in ../packages.scm and ../alt-structures.scm.
@@ -8,7 +10,7 @@
 ; structures that export privileged operations.
 
 (define-structure escapes escapes-interface
-  (open scheme-level-2 define-record-types simple-signals)
+  (open scheme-level-2 define-record-types low-exceptions)
   (files escape))
 
 (define-structures ((primitives primitives-interface)
@@ -20,17 +22,22 @@
 	define-record-types
 	bitwise	    ;Only for re-export
 	features    ;Only for re-export
-	simple-signals
+	low-exceptions
 	;; templates -- unneeded now?
 	)
   (files primitives
 	 weak
 	 contin))
 
-(define-structure code-quote (export (code-quote :syntax))
+; NB: Our syntax-rules implementation really wants a quote here that
+; preserves sharing.  Scheme 48 doesn't traditionally do this.
+(define-structure code-quotation (export (code-quote :syntax))
   (open scheme-level-2)
   (begin
     (define-syntax code-quote
       (lambda (e r c)
-	`(,(r 'quote) ,(cadr e))))))
+	e)))) ; this version should never get called
 
+(define-structure syntax-transformers syntax-transformers-interface
+  (open scheme-level-2)
+  (files "../rts/low-syntax"))
