@@ -1,5 +1,7 @@
 ; -*- Mode: Scheme; Syntax: Scheme; Package: Scheme; -*-
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber
 
 
 ; This is file pseudoscheme-features.scm.
@@ -10,21 +12,25 @@
 
 ; SIGNALS
 
-(define error #'ps:scheme-error)
+(define (error who message . irritants)
+  (apply #'ps:scheme-error message irritants))
 
-(define warn #'ps:scheme-warn)
+(define (assertion-violation who message . irritants)
+  (apply #'ps:scheme-error message irritants))
 
-(define (signal type . stuff)
-  (apply warn "condition signalled" type stuff))
+(define (implementation-restriction-violation who message . irritants)
+  (apply #'ps:scheme-error message irritants))
 
-(define (syntax-error . rest)		; Must return a valid expression.
-  (apply warn rest)
+(define (warning who message . irritants)
+  (apply #'ps:scheme-warn message irritants))
+
+(define (note who message . irritants)
+  (apply #'ps:scheme-warn message irritants))
+
+(define (syntax-violation who message form . maybe-subform)
+  (apply warning who message form maybe-subform)
   ''syntax-error)
 
-(define (call-error message proc . args)
-  (error message (cons proc args)))
-
-
 ; FEATURES
 
 (define force-output #'lisp:force-output)
