@@ -1,3 +1,6 @@
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Martin Gasbichler, Mike Sperber
 
 ; Optimizations are marked with +++, and may be flushed if desired.
 
@@ -90,7 +93,7 @@
         (if (pair? binding)
 	    (compile-local-name node name binding depth)
 	    (let ((offset (template-offset frame depth))
-		  (index (binding->index frame binding name value-type)))
+		  (index (binding->index frame binding name #f)))
 	      (instruction (enum op global)
 			   (high-byte offset)
 			   (low-byte offset)
@@ -106,7 +109,8 @@
 	  ((null? (cdr rest))
 	   (stack-indirect-instruction stack-offset (car rest)))
 	  (else
-	   (error "variable has too many indirections" name binding)))))
+	   (assertion-violation 'compile-local-name "variable has too many indirections"
+				name binding)))))
 
 ;----------------------------------------------------------------
 ; Hacked versions of the above for peephole optimization of pushes.
@@ -196,12 +200,12 @@
 		    (rest (cdr binding)))
 		(if (null? rest)		; in this frame
 		    (stack-set!-instruction stack-offset)
-		    (error "SET! on a closed-over variable" name)))
+		    (assertion-violation 'set! "SET! on a closed-over variable" name)))
 	      (let ((offset (template-offset frame depth))
 		    (index (binding->index frame
 					   binding
 					   name
-					   usual-variable-type)))
+					   #t)))
 		(instruction (enum op set-global!)
 			     (high-byte offset)
 			     (low-byte offset)
@@ -270,7 +274,8 @@
  	       (let ((primop (node-form proc-node)))
  		 (if (primop-compilator primop)
  		     ((primop-compilator primop) node depth frame cont)
- 		     (error "compiler bug: primop has no compilator"
+ 		     (assertion-violation 'compile-call
+					  "compiler bug: primop has no compilator"
  			    primop
  			    (schemify node)))))
 	      (else
@@ -311,7 +316,7 @@
 			     (+ depth nargs)
 			     frame
 			     (fall-through-cont node 0))
-		    (call-instruction nargs label)
+		    (call-instruction nargs (+ depth nargs) label)
 		    after))))
 
 ; A redex is a call of the form ((lambda (x1 ... xn) body ...) e1 ... en).
@@ -427,7 +432,7 @@
 
 (define (push-continuation depth frame cont node)
   (if (return-cont? cont)
-      (error "making a return point in tail position"))
+      (assertion-violation 'push-continuation "making a return point in tail position" cont))
   (let ((protocol (continuation-protocol (if (ignore-values-cont? cont)
                                              0
                                              1)
@@ -548,7 +553,7 @@
 ; signficantly slower (because the argument count cannot be encoded in
 ; the protocol).
 
-(define (call-instruction nargs label)
+(define (call-instruction nargs depth label)
   (if label
       (if (> nargs maximum-stack-args)				;+++
 	  (instruction-using-label (enum op big-call)
@@ -565,7 +570,9 @@
 		       (high-byte nargs)
 		       (low-byte nargs))
 	  (instruction (enum op tail-call)
-		       nargs))))
+		       nargs
+                       (high-byte depth)
+                       (low-byte depth)))))
   
 (define (stack-ref-instruction index)
   (if (>= index byte-limit)					;+++
@@ -678,6 +685,8 @@
 
 ; Find lookup result that was cached by classifier
 
+; The binding property is either a location, a 1- or 2-element list with
+; stack index and template index, or 'unbound
 (define (name-node-binding node)
   (or (node-ref node 'binding)
       (node-form node)))
@@ -685,7 +694,7 @@
 ; Produce something for source code that contains a compile-time error.
 
 (define (generate-trap depth frame cont . stuff)
-  (apply warn stuff)
+  (apply warning 'generate-trap stuff)
   (sequentially
     (stack-indirect-instruction (template-offset frame depth)
 				(literal->index frame (cons 'error stuff)))
@@ -722,9 +731,10 @@
 			  ((not (meet? proc-type any-procedure-type))
 			   ;; Could also check args for one-valuedness.
 			   (let ((message "non-procedure in operator position"))
-			     (warn message
-				   (schemify node)
-				   `(procedure: ,proc-type))
+			     (warning 'type-check
+				      message
+				      (schemify node)
+				      `(procedure: ,proc-type))
 			     (node-set! node 'type-error message))
 			   node)
 			  (else node)))
@@ -746,11 +756,12 @@
 		"argument type error")
 	       (else
 		"wrong number of arguments"))))
-    (warn message
-	  (schemify node)
-	  `(procedure wants:
-		      ,(rail-type->sexp (procedure-type-domain proc-type)
-					#f))
+    (warning 'diagnose-call-error
+	     message
+	     (schemify node)
+	     `(procedure wants:
+			 ,(rail-type->sexp (procedure-type-domain proc-type)
+					   #f))
 	  `(arguments are: ,(map (lambda (arg)
 				   (type->sexp (node-type arg) #t))
 				 (cdr (node-form node)))))
@@ -763,10 +774,3 @@
   (lambda (node depth frame cont)
     (compile (caddr (node-form node)) depth frame cont)))
 
-; Should this be in its own spot?
-
-(define-syntax receive
-  (syntax-rules ()
-    ((receive ?vars ?producer . ?body)
-     (call-with-values (lambda () ?producer)
-       (lambda ?vars . ?body)))))
