@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber
 
 ; Things used by the expression returned by REIFY-STRUCTURES.
 ; Cf. link/reify.scm.
@@ -35,8 +37,15 @@
     (make-table-immutable! (package-definitions p))
     p))
 
-(define (transform names+proc env type-exp source name)
-  (make-transform names+proc env (sexp->type type-exp #t) source name))
+(define (transform kind names+proc env type-exp source name)
+  (cond
+   ((eq? kind 'macro)
+    (make-transform/macro names+proc env (sexp->type type-exp #t) source name))
+   ((eq? kind 'inline)
+    (make-transform/inline names+proc env (sexp->type type-exp #t) source name))
+   (else
+    (assertion-violation 'transform
+			 "unknown transform kind" kind))))
 
 (define (package-define-static! package name static)
   (package-define! package
@@ -50,6 +59,7 @@
 			 ((structure? static)
 			  structure-type)
 			 (else
-			  (error "unknown kind of static value" static)))
+			  (assertion-violation 'package-define-static!
+					       "unknown kind of static value" static)))
 		   #f
 		   static))
