@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber, Taylor Campbell
 
 ; The entry point for all this.
 
@@ -19,32 +21,41 @@
 			    (get-location (thingie-binding x)
 					  package
 					  (thingie-name x)
-					  (thingie-want-type x))))
+					  (thingie-assigned? x))))
 	    ((template? x)
 	     (really-link! x package))))))
 
-; GET-LOCATION returns a location to be stored away in a template.  If the
-; wanted meta-type, which is either `value-type' or `usual-variable-type'
-; (for SET!), matches that in the binding, then we just return the binding's
-; location after noting that we are doing so.
+; GET-LOCATION returns a location to be stored away in a template.  If
+; ASSGINED? (which is #t if the variable is SET!) matches the type of
+; the binding, then we just return the binding's location after noting
+; that we are doing so.
 ;
 ; If the type doesn't match then we make a location and remember that we
 ; have done so.  If correct location becomes available later we will replace
 ; the bogus one (see env/pedit.scm).
 
-(define (get-location binding cenv name want-type)
-  (if (binding? binding)
-      (let ((place (binding-place binding)))
-	(cond ((compatible-types? (binding-type binding) want-type)
-	       (note-caching! cenv name place)
-	       place)
-	      ((variable-type? want-type)
-	       (get-location-for-unassignable cenv name))
-	      (else
-	       (warn "invalid variable reference" name cenv)
-	       (note-caching! cenv name place)
-	       place)))
-      (get-location-for-undefined cenv name)))
+(define (get-location binding cenv name assigned?)
+  (cond
+   ((binding? binding)
+    (let ((place (binding-place binding))
+	  (type (binding-type binding)))
+      (cond
+       ((variable-type? type)
+	(note-caching! cenv name place)
+	place)
+       (assigned?
+	(get-location-for-unassignable cenv name))
+       ((value-type? type)
+	(note-caching! cenv name place)
+	place)
+       (else
+	(warning 'get-location "invalid variable reference" name cenv)
+	(note-caching! cenv name place)
+	place))))
+    (assigned?
+     (get-location-for-undefined cenv name location-for-assignment))
+    (else
+     (get-location-for-undefined cenv name location-for-reference))))
 
 ; Packages have three tables used by env/pedit.scm:
 ;   undefineds
@@ -68,7 +79,7 @@
 (define get-undefined
   (location-on-demand package-undefineds))
 
-(define location-for-assignment
+(define get-undefined-but-assigned
   (location-on-demand package-undefined-but-assigneds))
 
 ; If this package is mutable and it gets NAME from some other structure
@@ -105,29 +116,33 @@
 
 ; Get a location for NAME which is SET! but isn't supposed to be.
 
+; In this case, we're creating a new, local location for the
+; assignment to happen - the original location isn't touched.
+
 (define (get-location-for-unassignable cenv name)
   (if (generated? name)
       (get-location-for-unassignable (generated-env name)
 				     (generated-name name))
       (let ((package (cenv->package cenv)))
-	(warn "invalid assignment" name)
+	(warning 'get-location-for-unassignable "invalid assignment" name)
 	(if (package? package)
-	    (lambda () (location-for-assignment package name))
+	    (lambda () (get-undefined-but-assigned package name))
 	    (lambda () (make-undefined-location name))))))
 
 ; Get a location for NAME, which is undefined.
 
-(define (get-location-for-undefined cenv name)
+(define (get-location-for-undefined cenv name get-location)
   (if (generated? name)
       (get-location-for-undefined (generated-env name)
-				  (generated-name name))
+				  (generated-name name)
+				  get-location)
       (let ((package (cenv->package cenv)))
 	((or (fluid $note-undefined)
 	     (lambda (cenv name) (values)))
 	   cenv
 	   name)
         (if (package? package)
-	    (let ((place (location-for-reference package name)))
+	    (let ((place (get-location package name)))
 	      (package-note-caching! package name place)
 	      place)
 	    (make-undefined-location name)))))
@@ -135,12 +150,11 @@
 (define $note-undefined (make-fluid (lambda (cenv name) (values))))
 
 ; Because of generated names CENV may not be an actual compile-env
-; (i.e. a procedure).
 
 (define (cenv->package cenv)
-  (cond ((procedure? cenv)
+  (cond ((compiler-env? cenv)
          ;; This returns #f if package is stable (static linking).
-         (extract-package-from-environment cenv))
+         (extract-package-from-comp-env cenv))
         ((package? cenv)
 	 cenv)
         ((structure? cenv)
@@ -149,18 +163,31 @@
 ; Exported for env/pedit.scm.
 
 (define (location-for-reference package name)
-  (let loop ((opens (package-opens package)))
-    (if (null? opens)
-	(get-undefined package name)
-	(call-with-values
-	    (lambda ()
-	      (interface-ref (structure-interface (car opens))
-			     name))
-	  (lambda (internal-name type)
-	    (if internal-name
-		(location-for-reference (structure-package (car opens))
-					internal-name)
-		(loop (cdr opens))))))))
+  (get-undefined-location package name get-undefined #f))
+
+(define (location-for-assignment package name)
+  (get-undefined-location package name get-undefined-but-assigned #t))
+
+(define (get-undefined-location package name get-undefined warn-assignment?)
+  (let location ((package package)
+		 (name name)
+		 (warn? warn-assignment?))
+    (let loop ((opens (package-opens package)))
+      (if (null? opens)
+	  (get-undefined package name)
+	  (call-with-values
+	      (lambda ()
+		(interface-ref (structure-interface (car opens))
+			       name))
+	    (lambda (internal-name type)
+	      (if internal-name
+		  (begin
+		    (if warn?
+			(warning 'location-for-assignment "invalid assignment" name))
+		    (location (structure-package (car opens))
+			      internal-name
+			      #f))
+		  (loop (cdr opens)))))))))
 
 ;----------------
 ; Maintain and display a list of undefined names.
@@ -205,7 +232,8 @@
 				(generated-name name)
 				name))
 			  (reverse names))))
-	  (apply warn
+	  (apply warning
+		 'print-undefined-names
 		 "undefined variables"
 		 env
 		 names)))))
