@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber
 
 
 ; Structures 'n' packages.
@@ -44,13 +46,15 @@
     (if (interface? int)
 	(begin (set-structure-interface! structure int)
 	       (note-reference-to-interface! int structure))
-	(call-error "invalid interface" initialize-structure! structure))))
+	(assertion-violation 'initialize-structure!
+			     "invalid interface" structure))))
 
 ; Make a structure over PACKAGE and the interface returned by INT-THUNK.
 
 (define (make-structure package int-thunk . name-option)
   (if (not (package? package))
-      (call-error "invalid package" make-structure package int-thunk))
+      (assertion-violation 'make-structure
+			   "invalid package" package int-thunk))
   (let ((struct (really-make-structure package
 				       (if (procedure? int-thunk)
 					   int-thunk
@@ -72,7 +76,8 @@
 	 (new-struct (make-structure (structure-package structure)
 				     (lambda ()
 				       (interface-maker
-				         (structure-interface structure))))))
+				         (structure-interface structure)))
+				    (structure-name structure))))
     (if (structure-unstable? structure)
 	(add-to-population! new-struct (structure-clients structure)))
     new-struct))
@@ -101,10 +106,10 @@
          (not (package-unstable? env)))
         ((structure? env)
          (not (structure-unstable? env)))
-        ((procedure? env)
+        ((compiler-env? env)
          #f)                    ; conservative
         (else
-         (error "invalid environment" env))))
+         (assertion-violation 'environment-stable? "invalid environment" env))))
 
 ; Map PROC down the the [name type binding] triples provided by STRUCT.
 
@@ -138,29 +143,42 @@
 		       clients
 		       unstable?
 		       integrate?
-		       file-name clauses loaded?)
+		       file-name reader clauses loaded?)
   package?
   (uid	           package-uid)
+  ;; #f if not initialized, then list of structures
   (opens           package-opens-really set-package-opens!)
+  ;; name-table name -> binding
   (definitions     package-definitions)
   (unstable?       package-unstable?)
+  ;; value of integrate clause; use integration in this packages
   (integrate?      package-integrate? set-package-integrate?!)
 
   ;; For EVAL and LOAD (which can only be done in unstable packages)
+  ;; package name -> location
   (get-location    package-get-location set-package-get-location!)
   (file-name       package-file-name)
+  (reader          package-reader set-package-reader!)
   (clauses         package-clauses)
   (loaded?         package-loaded? set-package-loaded?!)
+  ;; compiler environment
   (env             package->environment set-package->environment!)
 
   ;; For package mutation
   (opens-thunk     package-opens-thunk set-package-opens-thunk!)
+  ;; thunk -> (list (pair name struct))
   (accesses-thunk  package-accesses-thunk)
+  ;; locations introduced for missing values
+  ;; name-table name -> location
   (undefineds      package-real-undefineds set-package-undefineds!)
+  ;; locations introduced for missing cells
+  ;; name-table name -> location
   (undefined-but-assigneds
                    package-real-undefined-but-assigneds
 		   set-package-undefined-but-assigneds!)
   (clients         package-clients)
+  ;; locations used here that were supposed to have been provided by someone else
+  ;; name-table name -> place, see binding.scm
   (cached	   package-cached))
 
 (define-record-discloser :package
@@ -191,6 +209,7 @@
 	       unstable?		;unstable (suitable for EVAL)?
 	       #t			;integrate?
 	       file			;file containing DEFINE-STRUCTURE form
+	       read
 	       clauses			;misc. DEFINE-STRUCTURE clauses
 	       #f)))			;loaded?
     (note-package-name! new name)
@@ -204,6 +223,13 @@
   (make-compiler-env (lambda (name)
 		       (package-lookup package name))
 		     (lambda (name type . maybe-static)
+		       (cond
+			((and (symbol? name) ; generated names are hopefully of no interest here
+			      (opened-structure-for-name package name))
+			 => (lambda (struct)
+			      (warning 'package-define!
+				       "name from opened structure redefined"
+				       package name struct))))
 		       (package-define! package
 					name
 					type
@@ -214,6 +240,16 @@
 		     tower
 		     package))	; interim hack
 
+(define (opened-structure-for-name package name)
+  (let loop ((opens (package-opens-really package)))
+    (cond
+     ((null? opens)
+      #f)
+     ((structure-lookup (car opens) name #t)
+      (car opens))
+     (else
+      (loop (cdr opens))))))
+
 ; Two tables that we add lazily.
 
 (define (lazy-table-accessor slot-ref slot-set!)
@@ -269,7 +305,7 @@
 
 (define (make-simple-package opens unstable? tower . name-option)
   (if (not (list? opens))
-      (error "invalid package opens list" opens))
+      (assertion-violation 'make-simple-package "invalid package opens list" opens))
   (let ((package (make-package (lambda () opens)
 			       (lambda () '()) ;accesses-thunk
 			       unstable?
@@ -314,7 +350,8 @@
 			  (binding-type probe)
 			  (binding-place probe)
 			  static)
-	(error "internal error: name not bound" package name))))
+	(assertion-violation 'package-add-static!
+			     "internal error: name not bound" package name))))
 
 (define (package-refine-type! package name type)
   (let ((probe (table-ref (package-definitions package) name)))
@@ -323,7 +360,8 @@
 			  type
 			  (binding-place probe)
 			  (binding-static probe))
-	(error "internal error: name not bound" package name))))
+	(assertion-violation 'package-refine-type!
+			     "internal error: name not bound" package name))))
 
 ; --------------------
 ; Lookup
@@ -378,11 +416,11 @@
 	 (or (structure-lookup env
 			       name
 			       (package-integrate? (structure-package env)))
-	     (call-error "not exported" generic-lookup env name)))
-	((procedure? env)
+	     (assertion-violation 'generic-lookup "not exported" env name)))
+	((compiler-env? env)
 	 (lookup env name))
 	(else
-	 (error "invalid environment" env name))))
+	 (assertion-violation 'generic-lookup "invalid environment" env name))))
 
 ; --------------------
 ; Package initialization
@@ -390,6 +428,7 @@
 (define (initialize-package! package)
   (let ((opens ((package-opens-thunk package))))
     (set-package-opens! package opens)
+    (check-for-duplicates! package)
     (for-each (lambda (struct)
 		(if (structure-unstable? struct)
 		    (add-to-population! package (structure-clients struct))))
@@ -403,9 +442,35 @@
 			       (cdr name+struct)))
 	    (package-accesses package)))
 
+(define (check-for-duplicates! package)
+  (let ((imported-names (make-symbol-table)) ; maps names to pair of first binding, lists of structures
+	(duplicates '()))
+    (for-each (lambda (struct)
+		(for-each-export 
+		 (lambda (name type binding)
+		   (cond
+		    ((table-ref imported-names name)
+		     => (lambda (p)
+			  (if (not (same-denotation? (car p) binding))
+			      (begin
+				(set! duplicates (cons name duplicates))
+				(if (not (memq struct (cdr p)))
+				    (set-cdr! p (cons struct (cdr p))))))))
+		    (else
+		     (table-set! imported-names name (cons binding (list struct))))))
+		 struct))
+	      (package-opens package))
+    (for-each (lambda (duplicate)
+		(apply warning 'check-for-duplicates!
+		       "duplicate name in opened structure"
+		       duplicate
+		       package
+		       (cdr (table-ref imported-names duplicate))))
+	      duplicates)))
+
 ; (define (package->environment? env)
 ;   (eq? env (package->environment
-;	        (extract-package-from-environment env))))
+;	        (extract-package-from-comp-env env))))
 
 
 ; --------------------
