@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber
 
 ; Rudimentary type reconstruction, hardly worthy of the name.
 
@@ -212,10 +214,14 @@
     (examine (caddr (node-form node)) constrained value-type)
     unspecific-type))
 
-(define-reconstructor 'letrec syntax-type
-  (lambda (node constrained want-type)
-    (let ((form (node-form node)))
-      (reconstruct-letrec (cadr form) (caddr form) constrained want-type))))
+(let ((letrec-reconstructor
+       (lambda (node constrained want-type)
+	 (let ((form (node-form node)))
+	   (reconstruct-letrec (cadr form) (caddr form) constrained want-type)))))
+  (define-reconstructor 'letrec syntax-type
+    letrec-reconstructor)
+  (define-reconstructor 'letrec* syntax-type
+    letrec-reconstructor))
 
 (define-reconstructor 'pure-letrec syntax-type
   (lambda (node constrained want-type)
