@@ -1,4 +1,7 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber, Marcus Crestani, Robert Ransom,
+; Harald Glab-Phlak
 
 
 ; This code, along with C code in c/external.c, handles the interaction between
@@ -44,9 +47,9 @@
 
 (define-exported-binding "s48-stack-block-type" :stack-block)
 
-; There is no need to keep track of the VM frames.  These are all interchangable
+; There is no need to keep track of the VM frames.  These are all interchangeable
 ; because 1) the VM's state is kept in top-level variables and 2) we have
-; arranged it so that the relevent VM opcodes, call-external-value and
+; arranged it so that the relevant VM opcodes, call-external-value and
 ; return-from-callback, are all the same length and are always immediately
 ; followed by a return instruction.  s48_call_scheme can safely overwrite the
 ; template and code-pointer registers in the VM as they always point to a
@@ -93,8 +96,10 @@
 			  (dynamic-wind
 			   (lambda ()
 			     (if done?
-				 (error "attempt to throw into a callback"
-					(cons proc args))))
+				 (apply
+				  assertion-violation 'callback
+				  "attempt to throw into a callback"
+				  (cons proc args))))
 			   (lambda ()
 			     (let ((result (apply proc args)))
 			       (disable-interrupts!)
@@ -117,17 +122,17 @@
 ; Dealing with threads.
 ;
 ; The difficulty here is that each stack-block belongs to some thread.  Thread A
-; can call a C procedures which calls back into Scheme.  At that point a context
+; can call a C procedure which calls back into Scheme.  At that point a context
 ; switch occurs and we start running thread B, which promptly does the same
-; calls.  THe process stack then looks like this:
+; calls.  The process stack then looks like this:
 ; 
 ;  <C frame for VM running B1>
 ;  <C frame for s48_call_scheme>
-;  <C frame for B's c code>
+;  <C frame for B's C code>
 ;  <C frame for s48_external_call>
 ;  <C frame for VM running A1 and then B0>
 ;  <C frame for s48_call_scheme>
-;  <C frame for A's c code>
+;  <C frame for A's C code>
 ;  <C frame for s48_external_call>
 ;  <C frame for VM running A0>
 ;  <base>
@@ -168,25 +173,55 @@
 
 (define uncovered-return-handler
   (lambda (opcode reason . args)
-    (if (= reason (enum exception callback-return-uncovered))
-	(call-with-values
-	 (lambda ()
-	   (if (= 2 (length args))
-	       (values (car args)
-		       (cadr args)
-		       #f)
-	       (let ((args (reverse args)))
-		 (values (car args)
-			 (cadr args)
-			 (reverse (cddr args))))))
-	 (lambda (block return-value exception-args)
-	   (let ((placeholder (stack-block-placeholder block)))
-	     (set-stack-block-placeholder! block #f)
-	     (placeholder-set! placeholder #t)
-	     (if exception-args
-		 (apply signal-vm-exception opcode return-value exception-args)
-		 return-value))))
-	(apply signal-vm-exception opcode reason args))))
+
+    (define (blow-up con extract-message)
+      ;; look at external.c for why this is all so strangely reversed
+      (let ((rev (reverse args)))
+	(raise
+	 (condition
+	 con
+	 (make-external-exception)
+	 (make-who-condition (cadr rev))
+	 (make-message-condition
+	  (os-string->string (byte-vector->os-string (extract-message (car rev)))))
+	 (make-irritants-condition (reverse (cddr rev)))))))
+
+    (enum-case exception reason
+	       ((external-error)
+		(blow-up (make-error) values))
+	       ((external-assertion-violation)
+		(blow-up (make-assertion-violation) values))
+	       ((external-os-error)
+		(blow-up (make-error) os-error-message))
+	       ((out-of-memory)
+		(raise
+		 (condition
+		  (make-implementation-restriction-violation)
+		  (make-who-condition 'call-external-value)
+		  (make-message-condition "out of memory"))))
+	       ((callback-return-uncovered)
+		(call-with-values
+		    (lambda ()
+		      (if (= 2 (length args))
+			  (values (car args)
+				  (cadr args)
+				  #f)
+			  (let ((args (reverse args)))
+			    (values (car args)
+				    (cadr args)
+				    (reverse (cddr args))))))
+		  (lambda (block return-value exception-args)
+		    (let ((placeholder (stack-block-placeholder block)))
+		      (set-stack-block-placeholder! block #f)
+		      (placeholder-set! placeholder #t)
+		      (if exception-args
+			  (apply signal-vm-exception opcode return-value exception-args)
+			  return-value)))))
+	       (else
+		(apply signal-vm-exception opcode reason args)))))
+
+(define-condition-type &external-exception &serious
+  make-external-exception external-exception?)
 
 (define (block-depth block)
   (if block
@@ -210,8 +245,24 @@
 		   value
 		   (shared-binding-name proc)
 		   args)
-	    (apply call-error "bad procedure" call-imported-binding proc args)))
-      (apply call-error "bad procedure" call-imported-binding proc args)))
+	    (apply assertion-violation 'call-imported-binding "bad procedure"
+		   proc args)))
+      (apply assertion-violation 'call-imported-binding "procedure not defined"
+	     proc args)))
+
+(define (call-imported-binding-2 proc . args)
+  (if (and (shared-binding? proc)
+	   (shared-binding-is-import? proc))
+      (let ((value (shared-binding-ref proc)))
+	(if (byte-vector? value)
+	    (apply call-external-value-2
+		   value
+		   (shared-binding-name proc)
+		   args)
+	    (apply assertion-violation 'call-imported-binding-2 "bad procedure"
+		   proc args)))
+      (apply assertion-violation 'call-imported-binding-2 "procedure not defined"
+	     proc args)))
 
 ;----------------
 ; We export the record-type type so that external code can check to see if
