@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber
 
 
 ; Miniature command processor.
@@ -13,7 +15,7 @@
 	 (lambda (go)
 	   (with-handler
 	       (lambda (c punt)
-		 (cond ((or (error? c) (interrupt? c))
+		 (cond ((or (serious-condition? c) (interrupt-condition? c))
 			(display-condition c err)
 			(go (if batch?
 				(lambda () 1)
@@ -37,7 +39,8 @@
 			   (let ((form (read in)))
 			     (go (lambda ()
 				   (eval form (interaction-environment))))))
-			  (else (error "unknown command" (cadr form) 'go 'load (eq? (cadr form) 'load)))))
+			  (else (error 'command-processor
+				       "unknown command" (cadr form) 'go 'load (eq? (cadr form) 'load)))))
 		       (else
 			(call-with-values
 			    (lambda () (eval form (interaction-environment)))
