@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber
 
 
 ; Miniature package system.  This links mini-eval up to the output of
@@ -8,7 +10,7 @@
   (lambda (name)
     (let loop ((i (- (vector-length names) 1)))
       (if (< i 0)
-	  (error "unbound" name)
+	  (assertion-violation 'package "unbound" name)
 	  (if (eq? name (vector-ref names i))
 	      (contents (get-location (vector-ref locs i)))
 	      (loop (- i 1)))))))
@@ -26,7 +28,7 @@
 	  (cdr probe)
 	  (let loop ((opens opens))
 	    (if (null? opens)
-		(error "unbound" name)
+		(assertion-violation 'make-simple-package "unbound" name)
 		(if (memq name (structure-interface (car opens)))
 		    ((structure-package (car opens)) name)
 		    (loop (cdr opens)))))))))
