@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber
 
 
 ; These are the four entry points (cf. rts/eval.scm):
@@ -84,11 +86,11 @@
 		   (if (location-defined? loc)
 		       (contents loc)
 		       (error "uninitialized variable" (schemify node env))))
-		 (error "invalid variable reference" (schemify node env))))
+		 (assertion-violation 'name "invalid variable reference" (schemify node env))))
 	    ((unbound? binding)
-	     (error "unbound variable" (schemify node env)))
+	     (assertion-violation 'name "unbound variable" (schemify node env)))
 	    (else
-	     (error "peculiar binding" node binding))))))
+	     (assertion-violation 'name "peculiar binding" node binding))))))
 
 (define (name-node-binding node env)
   (or (node-ref node 'binding)
@@ -141,7 +143,7 @@
 
 (define (run-begin exp-list env)
   (if (null? exp-list)
-      (syntax-error "null begin")
+      (syntax-violation 'begin "null begin" `(begin ,@exp-list))
       (let loop ((exp-list exp-list))
 	(if (null? (cdr exp-list))
 	    (run (car exp-list) env)
@@ -158,9 +160,9 @@
 		      (variable-type? (binding-type probe)))
 		 (set-contents! (binding-place probe)
 				(run (caddr exp) env))
-		 (error "invalid assignment" (schemify node env))))
-	    ((unbound? probe) (error "unbound variable" exp))
-	    (else (error "peculiar assignment" exp))))))
+		 (assertion-violation 'set! "invalid assignment" (schemify node env))))
+	    ((unbound? probe) (assertion-violation 'set! "unbound variable" exp))
+	    (else (assertion-violation 'set! "peculiar assignment" exp))))))
 
 (define-interpreter 'if syntax-type
   (lambda (node env)
@@ -198,7 +200,8 @@
 
 
 (let ((bad (lambda (node env)
-	     (error "not valid in expression context" (node-form node)))))
+	     (assertion-violation 'definition
+				  "not valid in expression context" (node-form node)))))
   (define-interpreter 'define syntax-type bad)
   (define-interpreter 'define-syntax syntax-type bad))
 
@@ -210,7 +213,8 @@
     (let ((name (cadr (node-form node))))
       (or (table-ref primitive-procedures name)
 	  (lambda args
-	    (error "unimplemented primitive procedure" name))))))
+	    (assertion-violation 'primitive-procedure
+				 "unimplemented primitive procedure" name))))))
 
 (define primitive-procedures (make-table))
 
@@ -251,11 +255,11 @@
   (cond ((null? names)
 	 (if (null? args)
 	     env
-	     (error "too many arguments" args)))
+	     (assertion-violation 'bind-vars "too many arguments" args)))
 	((not (pair? names))
 	 (bind-var names args env))
 	((null? args)
-	 (error "too few arguments" names))
+	 (assertion-violation 'bind-vars "too few arguments" names))
 	(else
 	 (bind-var (car names) (car args)
 		   (bind-vars (cdr names) (cdr args) env)))))
