@@ -1,10 +1,12 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber
 
 
 ; ,config ,load debug/test.scm
 
 (define-structure testing (export (test :syntax) lost?)
-  (open scheme signals handle conditions)
+  (open scheme handle conditions)
   (open i/o)
   (begin
 
@@ -20,7 +22,7 @@
 	 (call-with-current-continuation
 	   (lambda (k)
 	     (with-handler (lambda (condition punt)
-			     (if (error? condition)
+			     (if (serious-condition? condition)
 				 (k condition)
 				 (punt)))
 	       thunk)))))
