@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber
 
 
 ; The user's state is in two parts:
@@ -93,10 +95,18 @@
 (define inspector-writing-length (user-context-accessor 'inspector-writing-length
 							(lambda () 5)))
 (define set-inspector-writing-length! (user-context-modifier 'inspector-writing-length))
-  
+
+(define condition-writing-depth (user-context-accessor 'condition-writing-depth
+						       (lambda () 5)))
+(define set-condition-writing-depth! (user-context-modifier 'condition-writing-depth))
+
+(define condition-writing-length (user-context-accessor 'condition-writing-length
+							(lambda () 6)))
+(define set-condition-writing-length! (user-context-modifier 'condition-writing-length))
+
 (define translations (user-context-accessor 'translations make-translations))
 (define set-translations! (user-context-modifier 'translations))
-
+  
 ;----------------
 ; User session state.
 ;
