@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber, Marcus Crestani
 
 
 ; Static linker.  Doesn't work very well this way (debug info is
@@ -21,7 +23,7 @@
 	filenames
 	debug-data		;with-fresh-compiler-state
 	locations
-	tables fluids simple-signals util)
+	tables fluids exceptions util)
   (files (link link)))
 
 (define-structure reification (export reify-structures)
@@ -34,7 +36,7 @@
 	locations
 	primops
 	tables records
-	simple-signals		;error
+	exceptions		;error
 	features		;force-output
 	util			;filter
 	inline)			;name->extrinsic
@@ -76,12 +78,14 @@
 (define-structure link-config (export )  ;dummy structure...
   (open scheme-level-2
 	linker
+	names ; name?
+	syntax-rules-apply
 	defpackage
 	types
 	analysis
 	loadc
 	flatloading
 	interfaces
-	simple-signals)    ;warn
+	exceptions)    ;warning
   ;; (files (alt init-defpackage.scm))  -- or (env ...), depending
   )
