@@ -1,4 +1,8 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber, Robert Ransom,
+; Marcel Turino, Manuel Dietrich, Marcus Crestani, Eric Knauel, 
+; Martin Gasbichler, Harald Glab-Phlak
 
 
 ; Interfaces for packages that can get loaded after the initial.image
@@ -14,7 +18,6 @@
 	  command-loop
 	  command-level-condition
 	  command-processor
-	  error-form ;foo
 	  execute-command
 	  exit-command-processor
 	  evaluate-and-select
@@ -25,11 +28,12 @@
 	  read-command			;take
 	  read-command-carefully	;inspect
 	  read-form
+	  with-sharp-sharp              ; ##
+	  current-sharp-sharp           ; ##
 	  run-sentinels
 	  set-command-results!
 	  start-command-processor
 	  restart-command-processor
-	  value->expression		;foo
 	  y-or-n?
 	  help
 	  define-command-syntax
@@ -62,7 +66,7 @@
 	  proceed-with-command-level
 	  kill-paused-thread!
 
-	  reset-command-input?  ; condition predicate 
+	  reset-command-input-condition?
 
 	  terminate-command-processor!
 	  command-level
@@ -93,6 +97,8 @@
 	  inspector-menu-limit set-inspector-menu-limit!
 	  inspector-writing-depth set-inspector-writing-depth!
 	  inspector-writing-length set-inspector-writing-length!
+	  condition-writing-depth set-condition-writing-depth!
+	  condition-writing-length set-condition-writing-length!
 	  translations set-translations!
 
 	  maybe-menu
@@ -148,6 +154,47 @@
    
 (define-interface profile-commands-interface
   (export profile))
+
+
+(define-interface profiler-interface
+  (export profile
+	  profile-thunk
+	  profile-count
+	  make-empty-profile-data
+
+	  ;; general data
+	  profile-data-starttime
+	  profile-data-endtime
+	  profile-data-memoryuse
+	  profile-data-gcruns
+	  profile-data-runtime
+	  profile-data-samples
+	  profile-data-templates
+	  profile-data-cycles
+	  profile-data-root
+
+	  ;; printing
+	  profile-display ; everything
+	  profile-display-overview
+	  profile-display-flat
+	  profile-display-tree
+  
+	  profile-display-function-flat
+	  profile-display-function-tree
+	  profile-display-function-cycle
+
+	  ;; flat accessors
+	  profile-function-calls
+	  profile-function-reccalls
+	  profile-function-nonreccalls
+	  profile-function-occurs
+	  profile-function-hist
+	  profile-function-memoryuse
+	  profile-function-timeshare
+	  profile-function-time-cumulative
+	  profile-function-time-self
+  ))
+
    
 (define-interface package-commands-interface
   (export in
@@ -162,7 +209,10 @@
 	  user-package-is
 	  config
 	  config-package-is
-	  undefine))
+	  undefine
+	  set-reader
+	  show-known-packages
+	  show-interface))
 
 (define-interface debug-commands-interface
   (export translate
@@ -190,7 +240,8 @@
 	  forget
 	  bound?
 	  expand
-	  expand-once))
+	  expand-once
+	  show-default-package))
 
 (define-interface usual-commands-interface
   (compound-interface
@@ -200,7 +251,6 @@
    debug-commands-interface
    inspect-commands-interface
    disassemble-commands-interface
-   ;profile-commands-interface
    ))
 
 (define-interface package-commands-internal-interface
@@ -213,10 +263,7 @@
 
 (define-interface display-conditions-interface
   (export display-condition		;command.scm
-	  &disclose-condition      	;env/disclosers.scm
-	  limited-write
-	  condition-display-depth set-condition-display-depth!
-	  condition-display-length set-condition-display-length!))
+	  limited-write))
 
 (define-interface debuginfo-interface
   (export read-debug-info
@@ -229,12 +276,12 @@
 	  error-form
 	  location-info
 	  location-name
+	  location-package-name
 	  template-debug-data
 	  template-id
 	  template-name
 	  template-names
-	  debug-data-names
-	  disclose-vm-condition))
+	  debug-data-names))
 
 (define-interface package-mutation-interface
   (export package-system-sentinel	;env/command.scm
@@ -242,15 +289,6 @@
 	  package-undefine!
 	  ))
 
-(define-interface packages-cruft-interface
-  (export assume-denotation
-	  ;; new-location-uid    ;?
-	  interface-ref
-	  structure-interface
-	  verify-package		;for debugging the package system
-	  ))
-
-
 ; --------------------
 ; Linker
 
@@ -276,7 +314,7 @@
 (define-interface extended-numbers-interface
   (export (define-extended-number-type :syntax)
 	  (define-opcode-extension :syntax)
-	  :exact :inexact
+	  <exact> <inexact>
 	  string-position
 	  &+
 	  &-
@@ -305,6 +343,7 @@
 	  &sin &cos &tan &asin &acos &atan1 &atan2
 	  &sqrt
 	  &make-rectangular
+	  &make-polar
 	  &number->string
 	  &really-string->number))
 
@@ -321,6 +360,8 @@
 	  general-category-id
 	  general-category-symbol
 	  general-category-primary-category
+	  general-categories
+	  general-category-index
 
 	  char-general-category
 	  char-titlecase
@@ -331,60 +372,6 @@
 	  string-foldcase
 	  string-titlecase))
 
-(define-interface conditions-interface
-  (export make-condition-type
-	  condition-type?
-	  make-condition
-	  condition?
-	  condition-has-type?
-	  condition-ref
-	  make-compound-condition
-	  extract-condition
-
-	  (define-condition-type :syntax)
-	  (condition :syntax)
-
-	  &condition
-	  &message message-condition? condition-message
-	  &serious serious-condition?
-	  &error error?
-	  &bug bug?
-	  &irritants irritants?
-	  condition-irritants
-	  &call-error call-error?
-	  call-error-proc call-error-args
-	  &vm-exception vm-exception?
-	  vm-exception-opcode vm-exception-reason vm-exception-arguments
-	  &warning warning?
-	  &note note?
-	  &syntax-error syntax-error?
-	  &interrupt interrupt?
-	  interrupt-type
-	  &decoding-error decoding-error?
-	  decoding-error-encoding-name
-	  &simple-condition simple-condition?
-	  simple-condition-type simple-condition-stuff))
-
-(define-interface i/o-conditions-interface
-  (export &i/o-error i/o-error?
-	  &i/o-port-error i/o-port-error?
-	  i/o-error-port
-	  &i/o-read-error i/o-read-error?
-	  &i/o-write-error i/o-write-error?
-	  &i/o-closed-error i/o-closed-error?
-	  &i/o-filename-error i/o-filename-error?
-	  i/o-error-filename
-	  &i/o-malformed-filename-error i/o-malformed-filename-error?
-	  &i/o-file-protection-error i/o-file-protection-error?
-	  &i/o-file-is-read-only-error i/o-file-is-read-only-error?
-	  &i/o-file-already-exists-error i/o-file-already-exists-error?
-	  &i/o-no-such-file-error i/o-no-such-file-error?
-	  &read-error read-error?
-	  read-error-line read-error-column read-error-position read-error-span
-
-	  &primitive-i/o-error primitive-i/o-error?
-	  i/o-error-status i/o-error-operation i/o-error-arguments))
-
 (define-interface signals-internal-interface
   (export simple-condition->condition
 	  condition->simple-condition
@@ -410,15 +397,154 @@
 	  string-normalize-nfc
 	  string-normalize-nfkc))
 
-(define-interface r6rs-unicode-interface
-  (compound-interface unicode-normalizations-interface
-		      (export char-titlecase
-			      char-title-case?
-			      char-foldcase
-			      string-upcase string-downcase
-			      string-foldcase
-			      string-titlecase)
-		      (export char-general-category)))
+; --------------------
+; Transport Link Cell Tables
+
+(define-interface tconc-queues-interface
+  (export make-tconc-queue
+          tconc-queue?
+          tconc-queue-empty?
+          tconc-queue-enqueue!
+          tconc-queue-dequeue!
+          tconc-queue-peek
+          tconc-queue-clear!
+          tconc-queue-size))
+
+(define-interface tlc-tables-interface
+  (export make-tlc-table
+          make-eq-tlc-table
+          make-eqv-tlc-table
+	  make-non-default-tlc-table
+          tlc-table?
+          tlc-table-size
+          tlc-table-ref
+          tlc-table-set!
+          tlc-table-delete!
+          tlc-table-contains?
+          tlc-table-update!
+          tlc-table-clear!
+	  tlc-table-resize!
+	  tlc-table-copy
+          tlc-table-keys
+          tlc-table-entries
+          tlc-table-equivalence-function
+          tlc-table-hash-function
+          tlc-table-has-tconc-queue?
+	  equal-hash string-hash string-ci-hash symbol-hash
+          tlc-table-distribution))
+
+; --------------------
+; Standards 
+
+; As R5RS is fixed, we duplicate much of the scheme-level-X-interfaces
+; to decouple them from scheme, which will change.
+(define-interface r5rs-interface
+  (export 
+   ;; from scheme-level-0-interface
+   ((if begin lambda letrec quote set!
+	define define-syntax let-syntax letrec-syntax)
+    :syntax)
+	  
+   ;; The basic derived expression types.
+   ((and cond do let let* or) :syntax)
+
+   apply
+
+   ;; Scalar primitives
+   eq?
+   number? integer? rational? real? complex?
+   exact? exact->inexact inexact->exact
+   + - * / = < > <= >=
+   quotient remainder
+   floor numerator denominator
+   real-part imag-part
+   exp log sin cos tan asin acos atan sqrt
+   angle magnitude make-polar make-rectangular
+   char?
+   char=? char<?
+   eof-object?
+   input-port? output-port?
+
+   ;; Data manipulation
+   pair? cons car cdr set-car! set-cdr!
+   symbol? symbol->string
+   string? make-string string-length string-ref string-set!
+   vector? make-vector vector-length vector-ref vector-set!
+
+   ;; Unnecessarily primitive
+   string=?
+   vector
+   assq
+
+   ;; New in Revised^5 Scheme
+   values call-with-values
+
+   ;; Things that aren't primitive at the VM level, but have
+   ;; to be defined using VM primitives.
+   string-copy
+   string->symbol
+   procedure?
+   integer->char char->integer
+
+   ;; from scheme-level-1-interface
+   ((case delay quasiquote syntax-rules) :syntax)
+   abs
+   append  assoc assv	  
+   boolean?
+   caaaar caaadr caadar caaddr caaar caadr caar
+   cadaar cadadr caddar cadddr cadar caddr cadr
+   cdaaar cdaadr cdadar cdaddr cdaar cdadr cdar
+   cddaar cddadr cdddar cddddr cddar cdddr cddr
+   char-alphabetic?
+   ceiling
+   char-ci<=? char-ci<? char-ci=? char-ci>=? char-ci>?
+   char-downcase char-lower-case? char-numeric?
+   char-upcase
+   char-upper-case? char-whitespace? char<=?
+   char>=? char>?
+   equal? eqv? even? expt
+   for-each force
+   gcd
+   inexact?
+   lcm length list list->string list->vector
+   list?				;New in R4RS
+   list-ref list-tail
+   map max member memq memv min modulo
+   negative? not null?
+   odd?
+   positive?
+   rationalize
+   reverse
+   round 
+   string string->list
+   string-append
+   string-ci<=? string-ci<? string-ci=? string-ci>=? string-ci>?
+   string-fill!
+   string<=? string<? string=? string>=? string>?
+   substring
+   truncate
+   vector->list vector-fill!
+   zero?
+
+   ;; from scheme-level-2-interface
+   (char-ready? (proc (&opt :input-port) :boolean))
+   call-with-current-continuation
+   call-with-input-file call-with-output-file
+   current-input-port current-output-port
+   dynamic-wind				;New in R5RS
+   close-input-port close-output-port
+   open-input-file open-output-file
+   with-input-from-file with-output-to-file
+   number->string string->number
+   read-char peek-char write-char
+   newline display write
+   read
+
+   ;; from scheme-adds-interface
+   eval load
+   interaction-environment
+   scheme-report-environment
+   null-environment))
 
 ; --------------------
 ; Big Scheme
@@ -466,6 +592,7 @@
 	  make-byte-vector-input-port make-string-input-port
 	  make-byte-vector-output-port make-string-output-port
 	  byte-vector-output-port-output string-output-port-output
+	  write-byte-vector-output-port-output write-string-output-port-output
 	  limit-output
 	  current-row current-column fresh-line
 
@@ -509,6 +636,10 @@
 	  inversion-list-cursor inversion-list-cursor-at-end?
 	  inversion-list-cursor-next inversion-list-cursor-ref))
 
+(define-interface constant-tables-interface
+  (export make-constant-table
+	  constant-table-lookup))
+
 (define-interface mask-types-interface
   (export make-mask-type
 	  mask-type?
@@ -534,8 +665,10 @@
 	  enum-set->list
 	  enum-set-member?
 	  enum-set=?
+	  enum-set-subset?
 	  enum-set-union
 	  enum-set-intersection
+	  enum-set-difference
 	  enum-set-negation))
 
 (define-interface enum-sets-internal-interface
@@ -543,7 +676,16 @@
 	  enum-set?
 	  enum-set-type
 	  enum-set->integer
-	  integer->enum-set))
+	  integer->enum-set
+
+	  ;; for r6rs-enums
+	  make-enum-set-type
+	  enum-set-type-values
+	  enum-set-type-member?
+	  elements->enum-set
+	  (define-enum-set-maker :syntax)
+	  enum-set-type-element-index
+	  ))
 
 (define-interface search-trees-interface
   (export make-search-tree
@@ -647,9 +789,28 @@
 	  placeholder-value
 	  placeholder-set!))
 
+(define-interface matchers-interface
+  (export matcher? matcher-sexpr
+	  matches?
+	  is
+	  anything
+	  opposite
+	  is-true is-false is-null
+	  is-within
+	  member-of
+	  all-of any-of list-where-all list-where-any
+	  list-of vector-of pair-of))
+
 (define-interface test-suites-interface 
   (export ((define-test-suite define-test-case define-test-cases) :syntax)
-	  ((check check-exception) :syntax)
-	  run-test-suite
+	  ((check check-terminates check-exception check-that check-exception-that) :syntax)
+	  run-test-suite run-test-cases
 	  =within
 	  zap-test-suite!))
+
+; Backwards compatibility
+
+(define-interface signals-interface
+  (export error warn syntax-error call-error note
+	  signal signal-condition
+	  make-condition))
