@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Ivan Shmakov, Mike Sperber
 
 
 ; This file contains things that tie together the compiler and the
@@ -55,8 +57,17 @@
 				  maybe-filename
 				  (package-uid package))))
     (link! template package note-undefined?)
-    (invoke-closure
-      (make-closure template
-		    (package-uid package)))))
+    (with-load-filename maybe-filename
+      (lambda ()
+	(invoke-closure
+	 (make-closure template
+		       (package-uid package)))))))
 
+(define $load-filename (make-fluid (make-cell #f)))
 
+(define (with-load-filename filename thunk)
+  (let-fluid $load-filename (make-cell filename)
+	     thunk))
+
+(define (current-load-filename)
+  (fluid-cell-ref $load-filename))
