@@ -1,5 +1,7 @@
 ; -*- Mode: Scheme; Syntax: Scheme; Package: Scheme; -*-
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber
 
 ;;;; Raising and handling of exceptions
 
@@ -34,21 +36,22 @@
     (lambda (k)
       (fluid-cell-set! $exception-handlers
 		       (list (last-resort-exception-handler k)))
+      (initialize-vm-exceptions! really-signal-condition)
       (thunk))))
 
 (define (last-resort-exception-handler halt)
   (let ((interrupt/keyboard (enum interrupt keyboard))
 	(losing? #f))
     (lambda (condition punt)
-      (cond ((error? condition)
+      (cond ((serious-condition? condition)
 	     (primitive-catch
 	       (lambda (c)
 		 (if (not losing?)
 		     (begin (set! losing? #t)
 			    (report-utter-lossage condition c)))
 		 (halt 123))))
-	    ((and (interrupt? condition)
-		  (= (cadr condition) interrupt/keyboard))
+	    ((and (interrupt-condition? condition)
+		  (= (interrupt-source condition) interrupt/keyboard))
 	     (halt 2))
 	    (else
 	     (unspecific))))))	;proceed
@@ -65,10 +68,15 @@
 			(enumerand->name (vm-exception-opcode condition)
 					 op)))
 	(else
-	 (apply debug-message
-		(condition-type condition)
-		" with no handler in place: "
-		(condition-stuff condition))))
+	 (call-with-values
+	     (lambda () (decode-condition condition))
+	   (lambda (type who message stuff)
+	     (apply debug-message
+		    type
+		    " [" who "]"
+		    " with no handler in place: "
+		    message
+		    stuff)))))
   (apply debug-message
 	 "stack template id's: "
 	 (map (lambda (id+pc)
@@ -85,7 +93,7 @@
   (call-with-current-continuation
     (lambda (k)
       (with-handler (lambda (c next)
-		      (if (error? c)
+		      (if (serious-condition? c)
 			  (k c)
 			  (next)))
 	thunk))))
@@ -100,16 +108,18 @@
 		      #f))))
     (if condition
 	(begin
-	  (apply warn message (append irritants (list condition)))
+	  (apply warning 'report-errors-as-warnings
+		 message
+		 (append irritants (list condition)))
 	  #t)
 	#f)))
 
 ; Define disclosers that are most important for error messages.
 
-(define-method &disclose ((obj :closure))
+(define-method &disclose ((obj <closure>))
   (list 'procedure (template-ref (closure-template obj) 1)))
 
-(define-method &disclose ((obj :location))
+(define-method &disclose ((obj <location>))
   (list 'location (location-id obj)))
 
 ; (put 'with-handler 'scheme-indent-hook 1)
@@ -122,10 +132,31 @@
      (handler condition))
    thunk))
 
-;; no tail recursive call to the handler here
 (define (raise obj)
   (let* ((cell (make-cell (fluid-cell-ref $exception-handlers)))
-	 (last-handler #f))
+	 (current-handler #f))
+    (let-fluid
+     $exception-handlers cell
+     (lambda ()
+       (let loop ((handlers (fluid-cell-ref $exception-handlers))
+		  (obj obj))
+	 (let ((rest (cdr handlers)))
+	   (cell-set! cell rest)
+	   (set! current-handler (car handlers))
+	   (current-handler obj
+			    (lambda maybe-obj
+			      (if (null? maybe-obj)
+				  (loop rest obj)
+				  (loop rest (car maybe-obj)))))))
+       (if (pair? (cell-ref cell))	; don't skip the last one
+	   (assertion-violation 'raise "exception handler returned"
+				current-handler obj))))
+    ;; go back to the top
+    (assertion-violation 'raise "exception handler returned"
+			 current-handler obj)))
+
+(define (raise-continuable obj)
+  (let* ((cell (make-cell (fluid-cell-ref $exception-handlers))))
     (let-fluid
      $exception-handlers cell
      (lambda ()
@@ -136,9 +167,7 @@
 	  (lambda maybe-obj
 	    (if (null? maybe-obj)
 		(loop (cdr handlers) obj)
-		(loop (cdr handlers) (car maybe-obj)))))
-	 (set! last-handler (car handlers)))))
-    (error "exception handler returned" last-handler obj)))
+		(loop (cdr handlers) (car maybe-obj))))))))))
 
 (define-syntax guard
   (syntax-rules ()
@@ -153,7 +182,7 @@
                   (lambda ()
                     (let ((var condition))      ; clauses may SET! var
                       (guard-aux (handler-k (lambda ()
-                                              (raise condition)))
+                                              (raise-continuable condition)))
                                  clause ...))))))))
           (lambda ()
             (call-with-values
@@ -191,3 +220,56 @@
      (if test
          (begin result1 result2 ...)
          (guard-aux reraise clause1 clause2 ...)))))
+
+(define (make-general-condition con who message irritants)
+  (apply condition
+	 con
+	 (make-message-condition message)
+	 (make-irritants-condition irritants)
+	 (if who
+	     (list (make-who-condition who))
+	     '())))
+
+(define (raise-general-trouble con who message irritants)
+  (raise (make-general-condition con who message irritants)))
+
+(define (error who message . irritants)
+  (raise-general-trouble (make-error)
+			 who message irritants))
+
+(define (assertion-violation who message . irritants)
+  (raise-general-trouble (make-assertion-violation)
+			 who message irritants))
+
+(define-syntax assert
+  (syntax-rules ()
+    ((assert ?exp)
+     (or ?exp
+	 (assertion-violation 'assert "assertion returned #f" '?exp)))))
+
+(define (implementation-restriction-violation who message . irritants)
+  (raise-general-trouble (make-implementation-restriction-violation)
+			 who message irritants))
+
+(define (warning who message . irritants)
+  (signal-condition
+   (make-general-condition (make-warning)
+			   who message irritants)))
+
+(define (note who message . irritants)
+  (signal-condition
+   (make-general-condition (make-note)
+			   who message irritants)))
+
+(define (syntax-violation who message form . maybe-subform)
+  (raise-general-trouble (make-syntax-violation form 
+						(and (pair? maybe-subform)
+						     (car maybe-subform)))
+			 who message '()))
+
+; Set LOW-EXCEPTIONS straight
+
+(initialize-low-exception-procedures!
+ error assertion-violation implementation-restriction-violation
+ warning note
+ syntax-violation)
