@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber, David Frese
 
 
 ; Interrupts
@@ -17,7 +19,9 @@
 	  (vector-set! handlers
 		       i
 		       (lambda stuff
-			 (apply signal (cons 'interrupt (cons i stuff))))))
+			 (signal-condition (condition
+					    (make-interrupt-condition (car stuff))
+					    (make-irritants-condition (cdr stuff)))))))
 	(vector-set! handlers
 		     (enum interrupt post-major-gc)
 		     (post-gc-handler #t spawn-on-root))
