@@ -1,4 +1,7 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber, Suresh Jagannathan, 
+; Henry Ceijtin
 
 ; Threads.
 
@@ -42,7 +45,7 @@
 (define-record-type thread :thread
   (really-make-thread dynamic-env dynamic-point proposal
 		      continuation scheduler
-		      cell arguments
+		      cell deadlock? arguments
 		      events current-task uid name)
   thread?
 
@@ -68,6 +71,10 @@
   ; Used by schedulers
   (data           thread-data             set-thread-data!)
   (cell           thread-cell             set-thread-cell!)
+  ; relevant if thread-cell is set (i.e. the thread is blocked); in
+  ; that case, it's #t if the thread contributed to deadlock, #f if it
+  ; doesn't
+  (deadlock?      thread-deadlock?	  set-thread-deadlock?!)
 
   ; For debugging
   (uid            thread-uid)       ; (also used as a cheap weak pointer)
@@ -92,6 +99,7 @@
 				     (thread-top-level thunk))
 				    #f 		     ; scheduler
 				    #f               ; cell
+				    #t               ; thread contributes to deadlock
 				    '()              ; arguments
 				    #f               ; events
 				    #f               ; current-task
@@ -245,19 +253,19 @@
   (let ((scheduler (current-thread)))
     (cond ((not (thread-continuation thread))
 	   (enable-interrupts!)
-	   (error "RUN called with a completed thread" thread))
+	   (assertion-violation 'run "RUN called with a completed thread" thread))
 	  ((not (eq? (thread-scheduler thread) scheduler))
 	   (enable-interrupts!)
-	   (error "thread run by wrong scheduler" thread scheduler))
+	   (assertion-violation 'run "thread run by wrong scheduler" thread scheduler))
 	  ((thread-cell thread)
 	   (enable-interrupts!)
-	   (error "thread run while still blocked" thread))
+	   (assertion-violation 'run "thread run while still blocked" thread))
 	  ((and (thread-current-task thread)
 		(not (null? (thread-arguments thread))))
 	   (enable-interrupts!)
-	   (error "returning values to running thread"
-		  thread
-		  (thread-arguments thread)))
+	   (assertion-violation 'run "returning values to running thread"
+				thread
+				(thread-arguments thread)))
 	  ((event-pending?)
 	   (enable-interrupts!)
 	   (apply values time (dequeue! (thread-events (current-thread)))))
@@ -428,7 +436,7 @@
 ; Another same thing, except that we have to be careful because we need to
 ; set the current thread's time field to a non-integer.
 
-(define (wait)
+(define (wait-for-event)
   (let ((interrupts (set-enabled-interrupts! no-interrupts)))
     (let ((thread (current-thread)))
       (if (not (and (thread-events thread)
@@ -454,16 +462,30 @@
 (define (block)
   (suspend (enum event-type blocked) '()))
 
+; Threads that are blocked, but are waiting for something that's not
+; another thread, but instead a signal, external event or some such.
+; Note that these threads must be made ready before they can be GCed.
+(define threads-not-deadlocked-count 0)
+
 ; Block if the current proposal succeeds.  Returns true if successful and false
 ; if the commit fails.  The cell becomes the thread's cell.  It will be cleared
 ; if the thread is killed.
 
-(define (maybe-commit-and-block cell)
+; The optional deadlock? argument (defaults to #t) says whether the
+; thread contributes to deadlock.
+
+(define (maybe-commit-and-block cell . maybe-deadlock?)
   (disable-interrupts!)
   (cond ((maybe-commit)
-	 (set-thread-cell! (current-thread) cell)
-	 (suspend-to (thread-scheduler (current-thread))
-		     (list (enum event-type blocked)))
+	 (let ((thread (current-thread)))
+	   (if (not (or (null? maybe-deadlock?)
+			(car maybe-deadlock?)))
+	       (begin
+		 (set-thread-deadlock?! thread #f)
+		 (set! threads-not-deadlocked-count (+ 1 threads-not-deadlocked-count))))
+	   (set-thread-cell! thread cell)
+	   (suspend-to (thread-scheduler thread)
+		       (list (enum event-type blocked))))
 	 #t)
 	(else
 	 (enable-interrupts!)
@@ -471,10 +493,10 @@
 
 ; Utility procedure for the common case of blocking on a queue.
 
-(define (maybe-commit-and-block-on-queue queue)
+(define (maybe-commit-and-block-on-queue queue . maybe-deadlock?)
   (let ((cell (make-cell (current-thread))))
     (enqueue! queue cell)
-    (maybe-commit-and-block cell)))
+    (apply maybe-commit-and-block cell maybe-deadlock?)))
 
 ; Send the upcall to the current scheduler and check the return value(s)
 ; to see if it was handled properly.
@@ -490,7 +512,8 @@
       (begin
 	(interrupt-thread thread
 			  (lambda ()
-			    (apply error "unhandled upcall" token args)))
+			    (apply error 'propogate-upcall "unhandled upcall"
+				   token args)))
 	(values))))
 
 (define (kill-thread! thread)   ; dangerous!
@@ -519,7 +542,7 @@
 	   (set-enabled-interrupts! interrupts))
 	  (else
 	   (set-enabled-interrupts! interrupts)
-	   (call-error "invalid argument" interrupt-thread thread)))))
+	   (assertion-violation 'interrupt-thread "invalid argument" thread)))))
   
 ;----------------
 ; Dealing with event queues
@@ -643,13 +666,18 @@
 (define (make-ready thread . args)
   (if (thread-cell thread)
       (begin
+	(if (not (thread-deadlock? thread))
+	    (begin
+	      (set! threads-not-deadlocked-count (- threads-not-deadlocked-count 1))
+	      (set-thread-deadlock?! thread #t)))
 	(clear-thread-cell! thread)
 	(set-thread-arguments! thread args)
 	(if (thread-scheduler thread)
 	    (schedule-event (thread-scheduler thread)
 			    (enum event-type runnable)
 			    thread)
-	    (error "MAKE-READY thread has no scheduler" thread)))))
+	    (assertion-violation 'make-ready
+				 "MAKE-READY thread has no scheduler" thread)))))
 
 (define (clear-thread-cell! thread)
   (let ((cell (thread-cell thread)))
@@ -738,14 +766,14 @@
     (lambda (exit-multitasking)
       (with-handler
        (lambda (c punt)
-	 (if (deadlock? c)
+	 (if (deadlock-condition? c)
 	     (exit-multitasking 0)
 	     (punt)))
        (lambda ()
 	 (call-with-current-continuation
 	   (lambda (terminate)
 	     (with-handler (lambda (c punt)
-			     (if (terminate? c)
+			     (if (terminate-condition? c)
 				 (terminate 0)
 				 (punt)))
 	       (lambda ()
@@ -768,25 +796,27 @@
 
 ; Raised when there is nothing to run.
 
-(define-condition-type 'deadlock '())
-(define deadlock? (condition-predicate 'deadlock))
+(define-condition-type &deadlock &serious
+  make-deadlock-condition deadlock-condition?)
 
 ; Raised when the current thread has been killed.
 
-(define-condition-type 'terminate '())
-(define terminate? (condition-predicate 'terminate))
+(define-condition-type &terminate &condition
+  make-terminate-condition terminate-condition?)
+
+(define the-terminate-condition (make-terminate-condition))
 
 ; Kill the current thread.  DEBUG-MESSAGE is used to try and make sure that some
 ; record exists when an error occured.  The system may be too broken for ERROR
 ; to work properly.
 
 (define (terminate-current-thread)
-  (signal 'terminate)
+  (signal-condition the-terminate-condition)
   (debug-message "Can't terminate current thread "
 		 (thread-uid (current-thread))
 		 " "
 		 (thread-name (current-thread)))
-  (error "can't terminate current thread")
+  (assertion-violation 'terminate-current-thread "can't terminate current thread")
   0)    ; suppress bogus compiler warning
 
 
