@@ -1,12 +1,15 @@
 ; -*- Mode: Scheme; Syntax: Scheme; Package: Scheme; -*-
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber, Robert Ransom
 
 
 ; This is file util.scm.
 
 ;;;; Utilities
 
-(define (unspecific) (if #f #f))
+; we import something equivalent to this from `primitives':
+; (define (unspecific) (if #f #f))
 
 (define (reduce cons nil list)		;used by length, append, etc.
   (if (null? list)
@@ -71,6 +74,9 @@
   (string->symbol (apply string-append
                          (map symbol->string syms))))
 
+(define (x->boolean x)
+  (if x #t #f))
+
 ;----------------
 ; Variations on a theme.
 ;
@@ -102,66 +108,8 @@
 	 (lambda (acc0 acc1 acc2)
 	   (loop (cdr list) acc0 acc1 acc2))))))
 
-;----------------
-; A version of LET and LET* which allows clauses that return multiple values.
-;
-; There is another copy of this in big/mvlet.scm.
-;
-; MV = multiple-value
-;
-; (mvlet (<clause> ...) <body>)
-; (mvlet* (<clause> ...) <body>)
-;
-; <clause> ::= (<ids> <expression>)
-; <ids> ::= <id> | (<id> ...) | (<id> ... . <id>)
-;
-; A clause of the form (<id> <exp>) is like a normal LET clause.  There is no
-; clause equivalent to
-;   (call-with-values (lambda () <expression>)
-;                     (lambda <id> <body>))
-
-(define-syntax mvlet
+(define-syntax receive
   (syntax-rules ()
-    ((mvlet () body ...)
-     (let () body ...))
-    ((mvlet (clause ...) body ...)
-     (mvlet-helper (clause ...) () (body ...)))))
-
-(define-syntax mvlet-helper
-  (syntax-rules ()
-    ((mvlet-helper () clauses (body ...))
-     (let clauses body ...))
-    ((mvlet-helper (((var . more-vars) val) more ...) clauses body)
-     (copy-vars (var . more-vars) () val (more ...) clauses body))
-    ((mvlet-helper ((var val) more ...) clauses body)
-     (mvlet-helper (more ...) ((var val) . clauses) body))))
-
-(define-syntax copy-vars
-  (syntax-rules ()
-    ((copy-vars (var . more-vars) (copies ...)
-		val more clauses body)
-     (copy-vars more-vars (copies ... x)
-		val more ((var x) . clauses) body))
-    ((copy-vars () copies val more clauses body)
-     (call-with-values
-       (lambda () val)
-       (lambda copies
-	 (mvlet-helper more clauses body))))
-    ((copy-vars last (copies ...) val more clauses body)
-     (call-with-values
-       (lambda () val)
-       (lambda (copies ... . lastx)
-	 (mvlet-helper more ((last lastx) . clauses) body))))))
-
-(define-syntax mvlet*
-  (syntax-rules ()
-    ((mvlet* () body ...)
-     (let () body ...))
-    ((mvlet* (((vars ...) val) clause ...) body ...)
-     (call-with-values
-       (lambda () val)
-       (lambda (vars ...)
-	 (mvlet (clause ...) body ...))))
-    ((mvlet* ((var val) clause ...) body ...)
-     (let ((var val)) (mvlet (clause ...) body ...)))))
-
+    ((receive ?vars ?producer . ?body)
+     (call-with-values (lambda () ?producer)
+       (lambda ?vars . ?body)))))
