@@ -1,6 +1,29 @@
 ; MODULE DEFINITION FOR SRFI-27, C/SCHEME-IMPLEMENTATION
 ; ======================================================
 ; 
+
+; Copyright (C) Sebastian Egner (2002). All Rights Reserved.
+; 
+; Permission is hereby granted, free of charge, to any person
+; obtaining a copy of this software and associated documentation
+; files (the "Software"), to deal in the Software without
+; restriction, including without limitation the rights to use, copy,
+; modify, merge, publish, distribute, sublicense, and/or sell copies
+; of the Software, and to permit persons to whom the Software is
+; furnished to do so, subject to the following conditions:
+; 
+; The above copyright notice and this permission notice shall be
+; included in all copies or substantial portions of the Software.
+; 
+; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+; EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+; MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+; NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+; BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+; ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+; CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+; SOFTWARE.
+
 ; Sebastian.Egner@philips.com, Mar-2002, in Scheme 48 0.57
 ;
 ; This file contains the top-level definition for the C-code
@@ -33,6 +56,8 @@
 ;   MG, September 2002: merged in mrg32k2a.scm, move package definitons to
 ;                       more-packages.scm, renamed from srfi-27-b.scm to srfi-27.scm
 
+(import-dynamic-externals "=scheme48external/srfi-27")
+
 (define-record-type :random-source
   (:random-source-make
    state-ref
@@ -51,15 +76,16 @@
 
 
 (define (:random-source-current-time)
-  (time-seconds (current-time)))
+  (current-time))
 
-					; interface to core generator
+;; interface to core generator
 
-(import-lambda-definition mrg32k3a-pack-state1 (state))
-(import-lambda-definition mrg32k3a-unpack-state1 (state))
-(import-lambda-definition mrg32k3a-random-range ())
-(import-lambda-definition mrg32k3a-random-integer (state range))
-(import-lambda-definition mrg32k3a-random-real (state))
+(import-lambda-definition-2 mrg32k3a-pack-state1 (state))
+(import-lambda-definition-2 mrg32k3a-unpack-state1 (state))
+(import-lambda-definition-2 mrg32k3a-random-range ())
+(import-lambda-definition-2 mrg32k3a-random-integer (state range))
+(import-lambda-definition-2 mrg32k3a-random-real (state))
+(import-lambda-definition-2 current-time ())
 
 (define (mrg32k3a-pack-state state)
   (mrg32k3a-pack-state1
