@@ -1,4 +1,7 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Marcus Crestani, David Frese,
+; Martin Gasbichler, Mike Sperber
 
 ; These are used to compile the VM separately from the GC.  It has new
 ; definitions of the heap and GC structures with all of the values defined
@@ -17,8 +20,14 @@
       (external "s48_allocate_tracedAgc" (=> (integer) address)))
     (define s48-allocate-untraced+gc
       (external "s48_allocate_untracedAgc" (=> (integer) address)))
+    (define s48-gc-can-allocate-unmovable?
+      (external "s48_gc_can_allocate_unmovableP" (=> () boolean)))
+    (define s48-allocate-traced-unmovable+gc
+      (external "s48_allocate_traced_unmovableAgc" (=> (integer) address)))
     (define s48-allocate-untraced-unmovable+gc
       (external "s48_allocate_untraced_unmovableAgc" (=> (integer) address)))
+    (define s48-unmovable?
+      (external "s48_unmovableP" (=> (integer) boolean)))
     ))
 
 (define-structure heap heap-interface
@@ -41,6 +50,8 @@
     (define s48-write-barrier
       (external "S48_WRITE_BARRIER" (=> (integer address integer) null)))
 
+    (define s48-check-heap
+      (external "s48_check_heap" (=> (integer) boolean)))
     (define s48-stob-in-heap?
       (external "s48_stob_in_heapP" (=> (integer) boolean)))
     ))
