@@ -1,3 +1,6 @@
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber
 
 (define (binary-flonum-op proc)
   (lambda (x y)
@@ -26,3 +29,10 @@
 				      (not (fl< y x)))))
 (define flonum>= (flonum-comparison (lambda (x y)
 				      (not (fl< x y)))))
+
+(define (flonum-rational? n)
+  (let ((x (extract-double n)))
+    ;; infinities and NaNs aren't rational
+    (and (fl= x x)
+	 (not (fl= x (fl/ 1.0 0.0)))
+	 (not (fl= x (fl/ -1.0 0.0))))))
