@@ -1,5 +1,7 @@
 ; -*- Mode: Scheme; Syntax: Scheme; Package: Scheme; -*-
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Marcus Crestani, Mike Sperber
 
 ; This manages external roots that can be registered and unregistered in a
 ; stack-like fashion.  It also provides the GC interface for the C FFI.
@@ -127,7 +129,8 @@
 (add-gc-root!
   (lambda () 
     (trace-external-root-stack)
-    (trace-permanent-external-roots)))
+    (trace-permanent-external-roots)
+    (trace-external-calls)))
 
 (define (trace-external-root-stack)
   (let loop ((frame *external-root-stack*))
