@@ -1,4 +1,8 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber, David Frese,
+; Martin Gasbichler
+
 ; Code for handling interrupts.
 
 ; New interrupt handler vector in *val*
@@ -171,11 +175,12 @@
           *os-signal-ring-start*)))
 
 (define (os-signal-ring-add! sig)
-  (os-signal-ring-inc! *os-signal-ring-end*)
-  (if (= *os-signal-ring-start*
-         *os-signal-ring-end*)
-      (error "OS signal ring too small, report to Scheme 48 maintainers"))
-  (vector-set! *os-signal-ring* *os-signal-ring-end* sig))
+  (let ((sig-pos *os-signal-ring-end*))
+    (os-signal-ring-inc! *os-signal-ring-end*)
+    (if (= *os-signal-ring-start*
+           *os-signal-ring-end*)
+        (error "OS signal ring too small, report to Scheme 48 maintainers"))
+    (vector-set! *os-signal-ring* sig-pos sig)))
 
 (define (os-signal-ring-empty?)
   (= *os-signal-ring-start*
@@ -185,11 +190,7 @@
   (if (os-signal-ring-empty?)
       (error "This cannot happen: OS signal ring empty"))
   (let ((sig (vector-ref *os-signal-ring* *os-signal-ring-start*)))
-    (set! *os-signal-ring-start* 
-          (if (= *os-signal-ring-start*
-                 (- *os-signal-ring-length* 1))
-              0
-              (+ *os-signal-ring-start* 1)))
+    (os-signal-ring-inc! *os-signal-ring-start*)
     sig))
 
 
@@ -398,9 +399,5 @@
 	 (error-message "unknown type of event")
 	 0)))
 
-; Return a bitmask for INTERRUPT.
-
-(define (interrupt-bit interrupt)
-  (shift-left 1 interrupt))
 
 
