@@ -1,5 +1,8 @@
 ; -*- Mode: Scheme; Syntax: Scheme; Package: Scheme; -*-
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Marcus Crestani, Mike Sperber,
+; David Frese, Martin Gasbichler
 
 ; Scalar primitives
 
@@ -22,7 +25,7 @@
   (fixnum->)
   (lambda (x)
     (if (scalar-value? x)
-	(goto return (scalar-value->char x))
+	(goto return (scalar-value->vm-char x))
 	(raise-exception wrong-type-argument 0 (enter-fixnum x)))))
 
 (define-primitive scalar-value?
@@ -41,7 +44,7 @@
 		 (> (+ start count) (code-vector-length buffer)))
 	     (raise-exception wrong-type-argument 0
 			      (enter-fixnum encoding)
-			      (scalar-value->char value)
+			      (scalar-value->vm-char value)
 			      buffer (enter-fixnum start) (enter-fixnum count))
 	     (call-with-values
 		 (lambda () 
@@ -56,13 +59,13 @@
 		     (raise-exception bad-option 0
 				      (enter-fixnum encoding)))))))))))
 
-(define-encode-char encode-char
+(define-encode-char char->utf
   (lambda (ok? out-of-space? count)
     (push (enter-boolean (and ok? (not out-of-space?))))
     (push (if ok? (enter-fixnum count) false))
     (goto return-values 2 null 0)))
 
-(define-encode-char encode-char!
+(define-encode-char char->utf!
   (lambda (ok? out-of-space? count)
     (goto return unspecific-value)))
 
@@ -89,22 +92,22 @@
 			 (lambda () (values ok? incomplete? value count))
 		       ?cont))))))))))
 
-(define-decode-char decode-char
+(define-decode-char utf->char
   (lambda (ok? incomplete? value count)
     (push (if (and ok? (not incomplete?))
-	      (scalar-value->char value)
+	      (scalar-value->vm-char value)
 	      false))
     (push (if ok? (enter-fixnum count) false))
     (goto return-values 2 null 0)))
 
 ; this makes limited sense: we only get the exception side effect
-(define-decode-char decode-char!
+(define-decode-char utf->char!
   (lambda (ok? incomplete? value count)
     (goto return unspecific-value)))
 
 (define-primitive eof-object?
   (any->)
-  (lambda (x) (vm-eq? x eof-object))
+  (lambda (x) (vm-eq? x vm-eof-object))
   return-boolean)
 
 ;----------------
@@ -272,33 +275,41 @@
 	   (raise-exception index-out-of-range 0
 			    vector (enter-fixnum index) (enter-elt char))))))
 
-(let ((proc (make-byte-setter vm-string-set! vm-string-length scalar-value->char)))
+(let ((proc (make-byte-setter vm-string-set! vm-string-length scalar-value->vm-char)))
   (define-primitive string-set! (string-> fixnum-> char-scalar-value->) proc))
 
 (let ((proc (make-byte-setter code-vector-set! code-vector-length enter-fixnum)))
   (define-primitive byte-vector-set! (code-vector-> fixnum-> fixnum->) proc))
 
-(define (byte-vector-maker size bytes type initialize setter enter-elt)
+(define (byte-vector-maker size bytes type initialize setter enter-elt
+			   unmovable?)
   (lambda (len init)
-    (let ((size (size len)))
-      (if (or (< len 0)
-	      (> size max-stob-size-in-cells))
-	  (raise-exception wrong-type-argument
-			   0
-			   (enter-fixnum len)
-			   (enter-elt init))
-	  (let ((vector (maybe-make-b-vector+gc type (bytes len))))
-	    (if (false? vector)
-		(raise-exception heap-overflow
-				 0
-				 (enter-fixnum len)
-				 (enter-elt init))
-		(begin
-		  (initialize vector len)
-		  (do ((i (- len 1) (- i 1)))
-		      ((< i 0))
-		    (setter vector i init))
-		  (goto return vector))))))))
+    ;; this test would be better placed in
+    ;; maybe-make-unmovable-b-vector+gc, but that would introduce a
+    ;; circular dependency:
+    (if (and unmovable? (not (s48-gc-can-allocate-unmovable?)))
+	(raise-exception unimplemented-instruction 0)
+	(let ((size (size len)))
+	  (if (or (< len 0)
+		  (> size max-stob-size-in-cells))
+	      (raise-exception wrong-type-argument
+			       0
+			       (enter-fixnum len)
+			       (enter-elt init))
+	      (let ((vector (if unmovable?
+				(maybe-make-unmovable-b-vector+gc type (bytes len))
+				(maybe-make-b-vector+gc type (bytes len)))))
+		(if (false? vector)
+		    (raise-exception heap-overflow
+				     0
+				     (enter-fixnum len)
+				     (enter-elt init))
+		    (begin
+		      (initialize vector len)
+		      (do ((i (- len 1) (- i 1)))
+			  ((< i 0))
+			(setter vector i init))
+		      (goto return vector)))))))))
 
 (let ((proc (byte-vector-maker vm-string-size
 			       scalar-value-units->bytes
@@ -306,16 +317,20 @@
 			       (lambda (string length)
 				 0)
 			       vm-string-set!
-			       scalar-value->char)))
+			       scalar-value->vm-char
+			       #f)))
   (define-primitive make-string (fixnum-> char-scalar-value->) proc))
   
-(let ((proc (byte-vector-maker code-vector-size
-			       (lambda (len) len)
-			       (enum stob byte-vector)
-			       (lambda (byte-vector length) 0)
-			       code-vector-set!
-			       enter-fixnum)))
-  (define-primitive make-byte-vector (fixnum-> fixnum->) proc))
+(let ((proc (lambda (unmovable?)
+	      (byte-vector-maker code-vector-size
+				 (lambda (len) len)
+				 (enum stob byte-vector)
+				 (lambda (byte-vector length) 0)
+				 code-vector-set!
+				 enter-fixnum
+				 unmovable?))))
+  (define-primitive make-byte-vector (fixnum-> fixnum->) (proc #f))
+  (define-primitive make-unmovable-byte-vector (fixnum-> fixnum->) (proc #t)))
 
 (define-primitive copy-string-chars! (string-> fixnum-> string-> fixnum-> fixnum->)
   (lambda (from from-index to to-index count)
@@ -358,12 +373,14 @@
     (make-immutable! thing)
     (goto return thing)))
 
-(define-primitive make-weak-pointer (any->)
-  (lambda (init)
-    (let ((weak-pointer (make-weak-pointer init weak-pointer-size)))
+(define-primitive make-weak-pointer ()
+  (lambda ()
+    (let ((weak-pointer (make-weak-pointer weak-pointer-size)))
+      (d-vector-init! weak-pointer 0 *val*)
       (goto continue-with-value
 	    weak-pointer
 	    0))))
+
 ;----------------
 ; Misc
 
@@ -373,7 +390,7 @@
 
 (define-primitive eof-object ()
   (lambda ()
-    (goto return eof-object)))
+    (goto return vm-eof-object)))
 
 (define-primitive trap (any->)
   (lambda (arg)
@@ -562,8 +579,26 @@
     (shared-set! *session-data* state))
   return-unspecific)
 
+; arg is either shared-binding name (for permanent event type) or #f
+(define-primitive new-external-event-uid (any->)
+  (lambda (arg)
+    (cond
+     ((shared-binding? arg)
+      (goto return-fixnum (permanent-external-event-uid arg)))
+     ((false? arg)
+      (goto return-fixnum (external-event-uid)))
+     (else
+      (raise-exception wrong-type-argument 0 arg)))))
+
+(define-primitive unregister-external-event-uid! (fixnum->)
+  (lambda (uid)
+    (unregister-external-event-uid! uid))
+  return-unspecific)
+
 ; Unnecessary primitives
 
+(define-primitive record-type<=? (record-type-> record-type->) record-type<=? return-boolean)
+
 (define-primitive string=? (string-> string->) vm-string=? return-boolean)
 
 ; Special primitive called by the reader.
@@ -579,7 +614,7 @@
                (i (- n 1) (- i 1)))
               ((< i 0)
 	       (goto return obj))
-            (vm-string-set! obj i (char->scalar-value (vm-car l))))))))
+            (vm-string-set! obj i (vm-char->scalar-value (vm-car l))))))))
 
 (define-primitive string-hash (string->) vm-string-hash return-fixnum)
 
