@@ -1,5 +1,7 @@
 ; -*- Mode: Scheme; Syntax: Scheme; Package: Scheme; -*-
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Mike Sperber, Martin Gasbichler
 
 ; The stack grows from higher addresses to lower ones.
 ; *STACK-BEGIN* and *STACK-END* delimit the stack portion of memory.
@@ -350,15 +352,15 @@
     (data-init! data exception-cont-instruction-size-index inst-size)
     (push-continuation! code-pointer)))
 
-(define (push-native-exception-continuation! code-pointer pc code exception)
-  (add-cells-to-stack! exception-continuation-cells)
+(define (push-native-exception-continuation! code-pointer exception 
+                                             bc-pc bc-code)
+  (add-cells-to-stack! native-exception-continuation-cells)
   (let ((data (address->stob-descriptor *stack*))
 	(size (enter-fixnum (operands-on-stack))))
-    (data-init! data exception-cont-size-index             size)
-    (data-init! data exception-cont-pc-index               pc)
-    (data-init! data exception-cont-code-index             code)
-    (data-init! data exception-cont-exception-index        exception)
-    (data-init! data exception-cont-instruction-size-index 0) ;empty
+    (data-init! data native-exception-cont-size-index      size)
+    (data-init! data native-exception-cont-exception-index exception)
+    (data-init! data native-exception-cont-bc-pc-index     bc-pc)
+    (data-init! data native-exception-cont-bc-code-index   bc-code)
     (push-continuation! code-pointer)))
 
 (define (pop-exception-data)
@@ -369,6 +371,13 @@
 	    (data-ref data exception-cont-exception-index)
 	    (data-ref data exception-cont-instruction-size-index))))
 
+(define (pop-native-exception-data)
+  (let ((data (address->stob-descriptor *stack*)))
+    (add-cells-to-stack! (- native-exception-continuation-cells))
+    (values (data-ref data native-exception-cont-exception-index)
+            (data-ref data native-exception-cont-bc-pc-index)
+            (data-ref data native-exception-cont-bc-code-index))))
+
 ; The indexes into the data are the indexes into the continuation minus
 ; the normal continuation cells.  We use raw STORE! and FETCH to avoid
 ; having to put a temporary header on the data (D-VECTOR-INIT! and
