@@ -1,4 +1,6 @@
-; Copyright (c) 1993-2008 by Richard Kelsey and Jonathan Rees. See file COPYING.
+; Part of Scheme 48 1.9.  See file COPYING for notices and license.
+
+; Authors: Richard Kelsey, Jonathan Rees, Timo Harter, Mike Sperber
 
 ; Disassembler that uses the VM's data structures.
 
@@ -67,9 +69,38 @@
       (display #\space)
       (write `(=> ,(+ start-pc (get-offset pc code)))))))
 
+
+; skip make-flat-env or make-big-flat-env
+; pc must point to the opcode, returns pc of next opcode
+;
+(define (skip-flat-env code pc)
+ (let* ((big?     (= (code-vector-ref code pc) (enum op make-big-flat-env)))
+	(size     (if big? 2 1))
+	(fetch    (if big? (lambda (code pc) (get-offset pc code)) code-vector-ref))
+	(offset   (+ pc 1)) ; skip opcode
+
+	(total    (fetch code offset))               ; # of values (total)
+	(closures (fetch code (+ offset size)))      ; # of closures
+	(offset   (+ offset size (* size closures))) ; skip template offset + offsets of templates in template
+
+	(vars     (fetch code offset))      ; # of vars in frame
+	(offset   (+ offset (* size vars))) ; skip offsets of vars in frame
+
+	(envs     (- total (+ closures vars))))  ; # of envs in frame
+    (let loop ((offset offset)                   ; skip env information
+               (env envs))
+      (if (= env 0)
+        offset  ; position of next opcode
+        (let ((env-vars (fetch code (+ offset size)))) ; # of vars in env
+          (loop 
+            (+ offset size size (* size env-vars)) ; skip env offset, # of vars, offsets of vars in env
+            (- env 1)))))))
+        
+
 (define (display-flat-env pc code)
   (let ((total-count (code-vector-ref code (+ pc 1))))
-    (display #\space) (write total-count) (display "...")))
+    (display #\space) (write total-count) (display " ...")
+    (skip-flat-env code pc)))
 
 ;    (let loop ((pc (+ pc 2)) (count 0) (old-back 0))
 ;      (if (= count total-count)
@@ -100,7 +131,7 @@
 		 (if (= pc 0) 5 4))
 		((= protocol args+nargs-protocol)
 		 (display "args+nargs")
-		 3)
+		 2)
 		((= protocol ignore-values-protocol)
 		 (display "discard all values")
 		 2)
@@ -119,7 +150,7 @@
 			   (display (list (if (= i 3) "3+" i)
 					  '=>
 					  (+ pc offset)))))))
-		 6)
+		 5)
 		(else
 		 (error "unknown protocol" protocol))))))
 
@@ -136,25 +167,25 @@
 
 (define (arg-spec-size spec)
   (case spec
-    ((nargs byte stob literal) 1)
-    ((offset small-index index two-bytes) 2)
+    ((nargs byte stob literal index stack-index) 1)
+    ((offset two-bytes two-byte-index two-byte-stack-index) 2)
     (else 0)))
 
 (define (print-opcode-arg specs pc code level write-templates?)
   (case (car specs)
-    ((nargs byte)
+    ((nargs byte stack-index)
      (write (code-vector-ref code pc)))
     ((literal)
      (write (- (code-vector-ref code pc) 128)))
-    ((two-bytes)
+    ((two-bytes two-byte-stack-index)
      (write (get-offset pc code)))
     ((index)
-     (write (get-offset pc code)))
-;     (let ((thing (template-ref template (get-offset pc code))))
-;       (write-literal-thing thing level write-templates?))
-    ((small-index)
-     (write (code-vector-ref pc code)))
+     (write (code-vector-ref code pc)))
 ;     (let ((thing (template-ref template (code-vector-ref code pc))))
+;       (write-literal-thing thing level write-templates?))
+    ((two-byte-index)
+     (write (code-vector-ref code pc)))
+;     (let ((thing (template-ref template (get-offset pc code))))
 ;       (write-literal-thing thing level write-templates?))
     ((offset)
      (write `(=> ,(+ pc -1 (get-offset pc code)))))  ; -1 to back up over opcode
